/*
 * Copyright 2017-2020 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.fortress.solver.constraint;

import java.util.ArrayList;
import java.util.Collection;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.Nodes;

/**
 * {@link FormulasEncoder} implements a trivial constraint encoder.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class FormulasEncoder implements ConstraintEncoder {

  private final Collection<Node> nodes;

  public FormulasEncoder() {
    this.nodes = new ArrayList<>();
  }

  public FormulasEncoder(final FormulasEncoder r) {
    this.nodes = new ArrayList<>(r.nodes);
  }

  @Override
  public void addNode(final Node node) {
    nodes.add(node);
  }

  @Override
  public Constraint encode() {
    return ConstraintUtils.newConstraint(Nodes.and(nodes));
  }

  @Override
  public FormulasEncoder clone() {
    return new FormulasEncoder(this);
  }
}
