package ru.ispras.fortress.transformer;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.expression.*;

public class IntToBitVecRuleSetTestCase {

  private static final NodeVariable leftIntArg = NodeVariable.newInteger("left");
  private static final NodeVariable rightIntArg = NodeVariable.newInteger("right");

  private static final NodeVariable leftBitVecArg = NodeVariable.newBitVector("left_bv", 32);
  private static final NodeVariable rightBitVecArg = NodeVariable.newBitVector("right_bv", 32);

  @Test
  public void mulTest() {
    final NodeOperation constraint = Nodes.mul(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVMUL);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void addTest() {
    final NodeOperation constraint = Nodes.add(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVADD);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void subTest() {
    final NodeOperation constraint = Nodes.sub(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSUB);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void divTest() {
    final NodeOperation constraint = Nodes.div(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSDIV);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void remTest() {
    final NodeOperation constraint = Nodes.rem(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSREM);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void modTest() {
    final NodeOperation constraint = Nodes.mod(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSMOD);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void lessEqTest() {
    final NodeOperation constraint = Nodes.lesseq(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSLE);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void greaterEqTest() {
    final NodeOperation constraint = Nodes.greatereq(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSGE);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void lessTest() {
    final NodeOperation constraint = Nodes.less(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSLT);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void greaterTest() {
    final NodeOperation constraint = Nodes.greater(leftIntArg, rightIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVSGT);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);

    final NodeVariable rightArg = (NodeVariable) transformedConstraint.getOperand(1);
    Assert.assertEquals(rightArg, rightBitVecArg);
  }

  @Test
  public void minusTest() {
    final NodeOperation constraint = Nodes.minus(leftIntArg);

    final NodeOperation transformedConstraint = (NodeOperation) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint.getOperationId(), StandardOperation.BVNEG);

    final NodeVariable leftArg = (NodeVariable) transformedConstraint.getOperand(0);
    Assert.assertEquals(leftArg, leftBitVecArg);
  }

  @Test
  public void plusTest() {
    final NodeOperation constraint = Nodes.plus(leftIntArg);

    final NodeVariable transformedConstraint = (NodeVariable) TypeConversion.castIntToBitVec(constraint, 32);

    Assert.assertEquals(transformedConstraint, leftBitVecArg);
  }
}
